/*
 * Copyright (C) 2007 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.example.android.snake;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import com.example.android.snake.models.AbstractObject;
import com.example.android.snake.models.GameWorld;
import com.example.android.snake.models.Lunatic;
import com.example.android.snake.models.Map;
import com.example.android.snake.models.Path;
import com.example.android.snake.models.Pillow;
import com.example.android.snake.util.Coordinate;

import android.app.Activity;
import android.content.Context;
import android.content.res.Resources;
import android.graphics.drawable.shapes.PathShape;
import android.hardware.Sensor;
import android.media.MediaPlayer;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

/**
 * SnakeView: implementation of a simple game of Snake
 * 
 *  
 */
public class SnakeView extends TileView implements Acceleration,SwipeObserver {

	private static final String TAG = "SnakeView";

	private GameWorld  world;
	
	public ImageView startScreen;
	
	// some variables for accelerator
	private AccelerometerTools acceleration;
	private Activity activity;
	private Coordinate lunatic;
	private Sound sound;
	private Sound sound2;
	private SoundMeter mic;
	private SwipeDetection swipe = new SwipeDetection();
	private List<Path> trace = new ArrayList<Path>(30);
	private List<Path> old_trace = new ArrayList<Path>(10);

	/**
	 * Current mode of application: READY to run, RUNNING, or you have already
	 * lost. static final ints are used instead of an enum for performance
	 * reasons.
	 */
	private int mMode = READY;
	public static final int PAUSE = 0;
	public static final int READY = 1;
	public static final int RUNNING = 2;
	public static final int LOSE = 3;
	
	/**
	 * Current direction the snake is headed.
	 */
	private int mDirection = NORTH;
	private int mNextDirection = NORTH;
	private static final int NOWHERE = 0;
	private static final int NORTH = 1;
	private static final int SOUTH = 2;
	private static final int EAST = 3;
	private static final int WEST = 4;
	private static final int NW = 5;
	private static final int NE = 6;
	private static final int SE = 7;
	private static final int SW = 8;

	/**
	 * Labels for the drawables that will be loaded into the TileView class
	 */
	public static final int LIFE = 1;
	public static final int LUNATIC = 2;
	public static final int WALL = 3;
	public static final int SPIDER = 4;
	public static final int PILLOW = 5;
	public static final int RFOOT = 6;
	public static final int LFOOT= 7;
	public static final int START_SCREEN= 8;
	
	public static final int[] TILE_SIZES = {
									0,
									64,
									128,
									64,
									128,
									64,
									64,
									64,
									1280
								}; 

	/**
	 * mScore: used to track the number of apples captured mMoveDelay: number of
	 * milliseconds between snake movements. This will decrease as apples are
	 * captured.
	 */
	private long mScore = 0;
	private long mMoveDelay = 25;
	/**
	 * mLastMove: tracks the absolute time when the snake last moved, and is
	 * used to determine if a move should be made based on mMoveDelay.
	 */
	private long mLastMove;

	/**
	 * mSnakeTrail: a list of Coordinates that make up the snake's body
	 * mAppleList: the secret location of the juicy apples the snake craves.
	 */
	private ArrayList<Coordinate> mSnakeTrail = new ArrayList<Coordinate>();
	private ArrayList<Coordinate> mAppleList = new ArrayList<Coordinate>();

	/**
	 * Everyone needs a little randomness in their life
	 */
	private static final Random RNG = new Random();

	/**
	 * Create a simple handler that we can use to cause animation to happen. We
	 * set ourselves as a target and we can use the sleep() function to cause an
	 * update/invalidate to occur at a later date.
	 */
	private RefreshHandler mRedrawHandler = new RefreshHandler();

	class RefreshHandler extends Handler {

		@Override
		public void handleMessage(Message msg) {
			SnakeView.this.update();
			SnakeView.this.invalidate();
		}

		public void sleep(long delayMillis) {
			this.removeMessages(0);
			sendMessageDelayed(obtainMessage(0), delayMillis);
		}
	};

	public void initStuff() {
		// swipe to throw pillow =)
		swipe.observers.add(this);
		
		// sound system start
		sound = new Sound(this.getContext());
		sound.playInGameMusic();
		sound = new Sound(this.getContext());

		// microphone start
		mic = new SoundMeter();
		mic.start();
	}

	/**
	 * Constructs a SnakeView based on inflation from XML
	 * 
	 * @param context
	 * @param attrs
	 */
	public SnakeView(Context context, AttributeSet attrs) {
		super(context, attrs);
		initStuff();
		initSnakeView();
	}

	public SnakeView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		initStuff();
		initSnakeView();
	}

	private void initSnakeView() {
		setFocusable(true);

		Resources r = this.getContext().getResources();

		resetTiles(TILE_SIZES.length);
		loadTile(LUNATIC, r.getDrawable(R.drawable.lunatic), 128);
		loadTile(SPIDER, r.getDrawable(R.drawable.spider), 128);
		loadTile(LIFE, r.getDrawable(R.drawable.life), 64);
		loadTile(WALL, r.getDrawable(R.drawable.wall), 64);
		loadTile(PILLOW, r.getDrawable(R.drawable.pillow), 64);
		loadTile(RFOOT, r.getDrawable(R.drawable.right_foot2), 64);
		loadTile(LFOOT, r.getDrawable(R.drawable.left_foot2), 64);
		loadTile(START_SCREEN, r.getDrawable(R.drawable.start), 1280);

		sound.resumeMusic();
	}

	private void initNewGame() {
	}

	/**
	 * Given a ArrayList of coordinates, we need to flatten them into an array
	 * of ints before we can stuff them into a map for flattening and storage.
	 * 
	 * @param cvec
	 *            : a ArrayList of Coordinate objects
	 * @return : a simple array containing the x/y values of the coordinates as
	 *         [x1,y1,x2,y2,x3,y3...]
	 */
	private double[] coordArrayListToArray(ArrayList<Coordinate> cvec) {
		int count = cvec.size();
		double[] rawArray = new double[count * 2];
		for (int index = 0; index < count; index++) {
			Coordinate c = cvec.get(index);
			rawArray[2 * index] = c.x;
			rawArray[2 * index + 1] = c.y;
		}
		return rawArray;
	}

	/**
	 * Save game state so that the user does not lose anything if the game
	 * process is killed while we are in the background.
	 * 
	 * @return a Bundle with this view's state
	 */
	public Bundle saveState() {
		Bundle map = new Bundle();

		map.putDoubleArray("mAppleList", coordArrayListToArray(mAppleList));
		map.putInt("mDirection", Integer.valueOf(mDirection));
		map.putInt("mNextDirection", Integer.valueOf(mNextDirection));
		map.putLong("mMoveDelay", Long.valueOf(mMoveDelay));
		map.putLong("mScore", Long.valueOf(mScore));
		if (lunatic != null) {
			map.putDouble("lunaticX", Double.valueOf(lunatic.x));
			map.putDouble("lunaticY", Double.valueOf(lunatic.y));
		}

		return map;
	}

	/**
	 * Given a flattened array of ordinate pairs, we reconstitute them into a
	 * ArrayList of Coordinate objects
	 * 
	 * @param rawArray
	 *            : [x1,y1,x2,y2,...]
	 * @return a ArrayList of Coordinates
	 */
	private ArrayList<Coordinate> coordArrayToArrayList(int[] rawArray) {
		ArrayList<Coordinate> coordArrayList = new ArrayList<Coordinate>();

		int coordCount = rawArray.length;
		for (int index = 0; index < coordCount; index += 2) {
			Coordinate c = new Coordinate(rawArray[index], rawArray[index + 1],
					0);
			coordArrayList.add(c);
		}
		return coordArrayList;
	}

	/**
	 * Restore game state if our process is being relaunched
	 * 
	 * @param icicle
	 *            a Bundle containing the game state
	 */
	public void restoreState(Bundle icicle) {
		setMode(PAUSE);

		mAppleList = coordArrayToArrayList(icicle.getIntArray("mAppleList"));
		mDirection = icicle.getInt("mDirection");
		mNextDirection = icicle.getInt("mNextDirection");
		mMoveDelay = icicle.getLong("mMoveDelay");
		mScore = icicle.getLong("mScore");
		lunatic.x = icicle.getInt("lunaticY");
		lunatic.y = icicle.getInt("lunaticX");
	}

	/**
	 * Updates the current mode of the application (RUNNING or PAUSED or the
	 * like) as well as sets the visibility of textview for notification
	 * 
	 * @param newMode
	 */
	public void setMode(int newMode) {
		int oldMode = mMode;
		mMode = newMode;

		if (newMode == RUNNING & oldMode != RUNNING) {
			startScreen.setVisibility(View.GONE);			
			sound.resumeMusic();
			update();
			return;
		}
			
		if (newMode == PAUSE) {
			startScreen.setVisibility(View.VISIBLE);
			sound.pauseMusic();
		}
		if (newMode == READY) {
			startScreen.setVisibility(View.VISIBLE);
			sound.resumeMusic();
		}
		if (newMode == LOSE) {
			startScreen.setVisibility(View.VISIBLE);
			sound.playCredits();
		}
	}


	
	public void updateLevel () {
		
		level = world.countEnemiesOnScreen();
		
		
		
		if (level != old_level) {
			Log.d(TAG, "Level now is:"+level);
			old_level = level;
			
			if (level > old_level) {
				sound.moreFear();
			} else {
				sound.lessFear();
			}
			
			switch (level) {
				case 1:
					world.setEnemiesLimit(10);
					break;
				case 2:
					world.setEnemiesLimit(15);
					break;
				case 3:
					world.setEnemiesLimit(25);
					break;
				case 4:
					world.setEnemiesLimit(35);
					break;
				case 5:
					world.setEnemiesLimit(50);
					break;
				case 6:
					world.setEnemiesLimit(80);
					break;
	
				default:
				break;
			}
		}
						
	}
	/**
	 * Handles the basic update loop, checking to see if we are in the running
	 * state, determining if a move should be made, updating the snake's
	 * location.
	 */
	private short fps_count = 0;
	private int trace_count = 0;	
	private int levelupdate_count = 0;
	private int last_foot = LFOOT;
	int level_switch_in= 10;
	int old_level = 0;
	int level = 0;
	public void update() {
		if (mMode == RUNNING) {
			long now = System.currentTimeMillis();

			if (now - mLastMove > mMoveDelay) {			
				clearTiles();
		

				if (world == null)
					initWorld();
								
				world.update();
				world.setScreen(mXOffset, mYOffset, displayWidth, displayHeight);

				// update lunatic
				world.updateLunatic(-(int) this.acceleration.getMyAzimuth());

				// trace
				if (mLastMove != 0)
					trace_count += now - mLastMove;
				
				for (Path t : trace) {
					setTile(t.tile_id, t.c, 64);
					t.TTL -= now - mLastMove;
					
					if(t.TTL < 0) 
						old_trace.add(t);
															
				}
				trace.removeAll(old_trace);
				
				if (trace_count > 900) {
					int foot = (last_foot == LFOOT)?RFOOT:LFOOT; 
					trace.add(new Path(world.getLunatic().getCoordinate(),foot));
					last_foot = foot; 
					trace_count = 0;
				}
				
				
				
				updateLevel();
				
				
				// draw map
				Map map = world.getMap();
				for (int x = 0; x < world.getMap().getX_size(); x++)
					for (int y = 0; y < world.getMap().getY_size(); y++)
						if (map.isWall(x*64, y*64))
							setTile(WALL, new Coordinate(x * 64, y * 64, 0), 64);

				// draw lunatic
				for (AbstractObject o : world.getAllObjects()) 
					drawObj(o);				
				
				mLastMove = now;
				
				now = System.currentTimeMillis();
				now -= now;
				mRedrawHandler.sleep((now < mMoveDelay) ? mMoveDelay - now : 0);
			}									
			
			levelupdate_count++;
			if (levelupdate_count == 1000*5) {
				updateLevel();
				levelupdate_count = 0;
			} 

		} else 		{
			setMode(RUNNING);		
		}
		

		double loudness = mic.getAmplitude();
		if (loudness > 8) {			
			Log.d(TAG, "Loudness: " + loudness);
		}

	}

	private Boolean phone_up = false;
	float phone_up_time = 0;

	public void sensorUpdate(float x, float y, float z, float sensorTimeStamp,
			float mCpuTimeStamp, int sensorType) {

		if (sensorType == Sensor.TYPE_ACCELEROMETER) {
			// Log.d("sensorUpdate","z= "+z);
			// Log.d("sensorUpdateTime","Timestamp: "+sensorTimeStamp);

			if (!phone_up) {
				if (z > 15) {
					phone_up = true;
					phone_up_time = sensorTimeStamp;
				}
			} else {
				if (z < -1) {
					phone_up = false;
					// Inside this, we have the jumping gesture!
					if (100000 < (sensorTimeStamp - phone_up_time)) {
						setMode(RUNNING);
						Log.d("sensorUpdate", "Jumping sleep walker!");
						world.getLunatic().jump = true;
					}
					phone_up_time = 0;
				}
			}

			float max = this.acceleration.getMaximumRange();
			float ycalib = (float) 0.043, xcalib = (float) -0.006, sensitivity = (float) 0.15;
			// tilted up
			if (((y / max) - ycalib) < -sensitivity) {
				// Log.d(TAG, "UP Tilt");
				if (mMode == READY | mMode == LOSE) {
					/*
					 * At the beginning of the game, or the end of a previous
					 * one, we should start a new game.
					 */
					initNewGame();
					setMode(RUNNING);
					update();					
					return;
				}

				if (mMode == PAUSE) {
					/*
					 * If the game is merely paused, we should just continue
					 * where we left off.
					 */
					setMode(RUNNING);
					update();
					return;
				}
			}
		}
	}

	/**
	 * connect Activity to this class
	 * 
	 * @param ac
	 */
	public void connectActivity(Activity ac) {
		this.activity = ac;
		acceleration = new AccelerometerTools(this, this.activity);
	}

	public void startAccelator() {
		acceleration.StartListening();
	}

	public void stopAccelator() {
		acceleration.StartListening();
	}

	public void pauseAccelator() {
		sound.pauseMusic();
		setMode(SnakeView.PAUSE);
		stopAccelator();
	}

	private void initWorld() {
		world = new GameWorld();
	}

	public void update(int[] vectors) {
		
		double angle;
		
		if(vectors[0] == 0) {
			if(vectors[1] > 0)
				angle = 270;
			else
				angle = 90;
		} else {
			angle = Math.atan(vectors[1] / vectors[0]);
			if (vectors[0] > 0)
				angle -= Math.PI;
		}			
		
		world.throwPillow((int) ((180/Math.PI)*angle));
		//play pillow sound
		
		
		Log.d(TAG,"Pillow throwed!");
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		setMode(RUNNING);
		return swipe.touchDetection(event);		
	}
		
}










































































