/*
 * Copyright (C) 2007 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.example.android.snake;

import java.util.HashMap;

import com.example.android.snake.models.AbstractObject;
import com.example.android.snake.util.Coordinate;



import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Display;
import android.view.View;
import android.view.WindowManager;
import android.view.Window;


/**
 * TileView: a View-variant designed for handling arrays of "icons" or other
 * drawables.
 * 
 */
public class TileView extends View {

    /**
     * Parameters controlling the size of the tiles and their range within view.
     * Width/Height are in pixels, and Drawables will be scaled to fit to these
     * dimensions. X/Y Tile Counts are the number of tiles that will be drawn.
     */

    protected static int mTileSize;

    protected static int mXTileCount=3840;
    protected static int mYTileCount=3840;

    public static int mXOffset = 0;
    public static int mYOffset = 0;
    
    public static int displayHeight;
    public static int displayWidth;
    
    private HashMap<Coordinate, Integer> tiles = new HashMap<Coordinate, Integer>();


    /**
     * A hash that maps integer handles specified by the subclasser to the
     * drawable that will be used for that reference
     */
    private Bitmap[] mTileArray; 


    private final Paint mPaint = new Paint();

    public TileView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);

        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.TileView);

        mTileSize = a.getInt(R.styleable.TileView_tileSize, 1);
        
        DisplayMetrics displaymetrics = new DisplayMetrics();
        ((WindowManager) getContext().getSystemService(Context.WINDOW_SERVICE))
        .getDefaultDisplay().getMetrics(displaymetrics);
        displayHeight = displaymetrics.heightPixels;
        displayWidth = displaymetrics.widthPixels;
        System.out.println("Display "+displayWidth+"x"+displayHeight);                
        
        a.recycle();
    }

    public TileView(Context context, AttributeSet attrs) {
        super(context, attrs);

        TypedArray a = context.obtainStyledAttributes(attrs, R.styleable.TileView);

        //mTileSize = a.getInt(R.styleable.TileView_tileSize, 12);
        mTileSize = a.getInt(R.styleable.TileView_tileSize, 1);
        
        DisplayMetrics displaymetrics = new DisplayMetrics();
        ((WindowManager) getContext().getSystemService(Context.WINDOW_SERVICE))
        .getDefaultDisplay().getMetrics(displaymetrics);
        displayHeight = displaymetrics.heightPixels;
        displayWidth = displaymetrics.widthPixels;
        System.out.println("Display "+displayWidth+"x"+displayHeight);  
         
        a.recycle();
    }

    
    
    /**
     * Rests the internal array of Bitmaps used for drawing tiles, and
     * sets the maximum index of tiles to be inserted
     * 
     * @param tilecount
     */
    
    public void resetTiles(int tilecount) {
    	mTileArray = new Bitmap[tilecount];
    }


    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh) {
//        mXOffset = displayWidth/2;
//        mYOffset = displayHeight/2;
        
        clearTiles();
    }

    public void clearTiles() {
		tiles.clear();
		
	}

	/**
     * Function to set the specified Drawable as the tile for a particular
     * integer key.
     * 
     * @param key
     * @param tile
     */
    public void loadTile(int key, Drawable tile, int tile_size) {
        //Bitmap bitmap = Bitmap.createBitmap(mTileSize, mTileSize, Bitmap.Config.ARGB_8888);
    	Bitmap bitmap = Bitmap.createBitmap(tile_size, tile_size, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        //tile.setBounds(0, 0, mTileSize, mTileSize);
        tile.setBounds(0, 0, tile_size, tile_size);
        tile.draw(canvas);
        
        mTileArray[key] = bitmap;
    }

    
    public void drawObj(AbstractObject o) {
    	setTile(o.getTileId(),o.getCoordinate(),64);
    }

    /**
     * Used to indicate that a particular tile (set with loadTile and referenced
     * by an integer) should be drawn at the given x/y coordinates during the
     * next invalidate/draw cycle.
     * 
     * @param tileindex
     * @param x
     * @param y
     */
    public void setTile(int tileindex, Coordinate c, int tile_size) {    	
    	if(isOnScreen(c.x,c.y)) {
    		tiles.put(c, tileindex);
    		if(tileindex == SnakeView.LUNATIC) 
    			setOffset((int)c.x-displayWidth/2,(int)c.y-displayHeight/2);
    	}
    } 

    private void setOffset(int x, int y) {
    	if (x > 0)
    		mXOffset = x;
    	if (y > 0)
    		mYOffset = y;		
	}

	private boolean isOnScreen(double x, double y) {
    	int magic = 128;
    	
    	if (x - mXOffset < -magic) 
    		return false;
    	if (y - mYOffset< -magic) 
    		return false;
    	if (x - mXOffset > displayWidth+magic) 
    		return false;
    	if (y - mYOffset > displayHeight+magic) 
    		return false;
    	
    	return true;
    }
    
    @Override  
    public void onDraw(Canvas canvas) {
        super.onDraw(canvas);              
                
        for (Coordinate c : tiles.keySet()){
        	canvas.save(); 
        	
        	if(tiles.get(c) == SnakeView.LUNATIC ||
    			tiles.get(c) == SnakeView.SPIDER ||
    			tiles.get(c) == SnakeView.PILLOW ||
    			tiles.get(c) == SnakeView.LFOOT ||
    			tiles.get(c) == SnakeView.RFOOT) 
        		canvas.rotate(c.angle,
        				(int)c.x - mXOffset,
        				(int)c.y - mYOffset);
        	
        	if(tiles.get(c) == SnakeView.WALL){
        		canvas.drawBitmap(mTileArray[tiles.get(c)], 
            			(int)c.x - mXOffset,
            			(int)c.y - mYOffset,
                		mPaint);
        	} else {
        		canvas.drawBitmap(mTileArray[tiles.get(c)], 
            			(int)c.x - mXOffset -SnakeView.TILE_SIZES[tiles.get(c)]/2 ,
            			(int)c.y - mYOffset -SnakeView.TILE_SIZES[tiles.get(c)]/2,
                		mPaint);
        	}
        	
        	canvas.rotate(c.angle);
        	canvas.restore();
        }
    }

}









































